import pygame


class Window:
    def __init__(self, size):
        self.surface = pygame.display.set_mode(size)

    @property
    def left(self):
        return self.surface.get_rect().left

    @property
    def right(self):
        return self.surface.get_rect().right

    @property
    def top(self):
        return self.surface.get_rect().top

    @property
    def bottom(self):
        return self.surface.get_rect().bottom

    @property
    def width(self):
        return self.surface.get_rect().width

    @property
    def height(self):
        return self.surface.get_rect().height

    @property
    def center(self):
        return self.surface.get_rect().center

    @property
    def centerx(self):
        return self.surface.get_rect().centerx

    @property
    def centery(self):
        return self.surface.get_rect().centery

    def set_caption(self, caption):
        self.caption = caption
        pygame.display.set_caption(caption)



class Player:
    def __init__(self, score=0):
        self.score = score
        self.objects = []

    def update(self):
        for obj in self.objects:
            obj.update()

    def draw(self, surface_to_draw_on):
        for obj in self.objects:
            obj.draw(surface_to_draw_on)

    def scored(self, points=1):
        self.score += points



import sys
from collections import defaultdict

import pygame


class Menu:
    def __init__(self, screen, fps=15):
        self.active = False
        self.screen = screen
        self.fps = fps
        self.clock = pygame.time.Clock()
        self.handlers = defaultdict(list)
        self.objects = []

    def handle_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            else:
                for handler in self.handlers[event.type]:
                    handler(event)

    def update(self):
        for obj in self.objects:
            obj.update()

    def draw(self):
        for obj in self.objects:
            obj.draw(self.screen.surface)

    def activate(self):
        self.active = True

    def deactivate(self):
        self.active = False

    def take_over(self):
        self.screen.surface.fill((180, 180, 180))
        self.handle_events()
        self.update()
        self.draw()

        pygame.display.update()
        self.clock.tick(self.fps)



class Object:
    def __init__(self, rect, speed=(0, 0)):
        self.rect = rect
        self.speed = speed

    @property
    def left(self):
        return self.rect.left

    @property
    def right(self):
        return self.rect.right

    @property
    def top(self):
        return self.rect.top

    @property
    def bottom(self):
        return self.rect.bottom

    @property
    def width(self):
        return self.rect.width

    @property
    def height(self):
        return self.rect.height

    @property
    def center(self):
        return self.rect.center

    @property
    def centerx(self):
        return self.rect.centerx

    @property
    def centery(self):
        return self.rect.centery

    def move(self, speed):
        dx, dy = speed
        self.rect = self.rect.move(dx, dy)

    def update(self):
        if self.speed != [0, 0]:
            self.move(self.speed)

    def draw(self, surface_to_draw_on):
        pass



import sys
from collections import defaultdict

import pygame

from .Window import Window


class Game:
    def __init__(self, fps=60, window_size=(0, 0), bg_color=(180, 180, 180)):
        self.game_over = False
        self.window = Window(window_size)
        self.fps = fps
        self.bg_color = bg_color
        self.clock = pygame.time.Clock()
        self.handlers = defaultdict(list)
        self.players = []
        self.objects = []
        self.menus = []

    def handle_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            else:
                for handler in self.handlers[event.type]:
                    handler(event)

    def update(self):
        for menu in self.menus:
            while menu.active:
                menu.take_over()
        for obj in self.objects:
            obj.update()
        for player in self.players:
            player.update()

    def draw(self):
        for obj in self.objects:
            obj.draw(self.window.surface)
        for player in self.players:
            player.draw(self.window.surface)

    def run(self):
        while not self.game_over:
            self.window.surface.fill(self.bg_color)

            self.handle_events()
            self.update()
            self.draw()

            pygame.display.update()
            self.clock.tick(self.fps)



import pygame

from .Object import Object


class Button(Object):
    def __init__(self, rect, color=(180, 180, 180), text=""):
        super().__init__(rect)
        pygame.font.init()
        self.color = color
        self.text = text
        self.font = pygame.font.SysFont("Sans Serif", 20, bold=True)

    def draw(self, surface_to_draw_on):
        text = self.font.render(self.text, False, (0, 0, 0))
        pygame.draw.rect(surface_to_draw_on, self.color, self.rect)
        surface_to_draw_on.blit(text, (self.rect.x + 10, self.rect.y + 10))
